import { each, includes, keys } from 'lodash'
import {
  documentHelpers,
  Tool
} from 'substance'

class EditNoteTool extends Tool {

  render ($$) {
    const selected = this.getSelection()
    const provider = this.getProvider()

    let el = $$('div').addClass('sc-edit-note-tool')

    if (!selected.node) return el
    
    provider.config.miniEditorContext.editor.emit('noteSelected', 'paragraph-71bf75436a5f9b56700064c6ee2116ab')
    return el
  }

  didMount () {
    this.context.editorSession.onUpdate('', this.disableTools, this)
  }

  disableTools () {
    const selected = this.getSelection()
    if (!selected.node) return
    const commandStates = this.context.commandManager.commandStates
    each(keys(commandStates), (key) => {
      const allowed = ['comment', 'redo', 'save', 'switch-text-type', 'undo', 'note']
      if (!includes(allowed, key)) commandStates[key].disabled = true
    })
    this.rerender()
  }

  getSelection () {
    // TODO -- write cleaner
    const surface = this.context.surfaceManager.getFocusedSurface()
    if (!surface) return {}

    const session = this.context.editorSession
    const sel = session.getSelection()

    const notes = documentHelpers.getPropertyAnnotationsForSelection(
      session.getDocument(),
      sel,
      { type: 'note' }
    )

    const note = notes[0]
    let show = false

    if (typeof note !== 'undefined') {
      if ((sel.start.offset === note.start.offset &&
        sel.end.offset === note.end.offset)) {
        show = true
      }
    }

    // disable when larger selection that just includes a note as well
    // const selectionLength = (sel.end.offset - sel.start.offset === 1)
    // if (sel.end.offset - sel.)

    if (show) {
      return {
        node: note
      }
    } else {
      return {
        node: null
      }
    }
  }

  getProvider () {
    return this.context.notesProvider
  }

  getSurface () {
    const surfaceManager = this.context.surfaceManager
    return surfaceManager.getFocusedSurface()
  }

  isEditorReadOnly () {
    const surface = this.getSurface()
    return surface.isReadOnlyMode()
  }
}

EditNoteTool.type = 'edit-note'

export default EditNoteTool
