import _ from 'lodash'

import { TOCProvider } from 'substance'

class NotesProvider extends TOCProvider {
  computeEntries () {
    const docMain = this.getDocument()
    const nodesMain = docMain.getNodes()

    const docMini = this.config.miniEditorSession.document
    let nodesMini = ''
    if (docMini) nodesMini = docMini.getNodes()

    if (docMini)console.log('paragraphs', nodesMini)

    // get all notes from the document
    const notesMain = _.pickBy(nodesMain, function (value, key) {
      return value.type === 'note'
    })
    const notesMini = _.pickBy(nodesMini, function (value, key) {
      return value.type === 'isolated-note'
    })

    const entriesMain = this.sortNodes(notesMain)
    const entriesMini = this.sortNodesMini(notesMini)

    // console.log(entriesMain, entriesMini)
    const entries = {
      main: entriesMain,
      mini: entriesMini
    }

    return entries
  }

  sortNodes (nodes) {
    let notes = _.clone(nodes)
    const doc = this.getDocument()
    const container = doc.get('body')

    // sort notes by
    //   the index of the containing block
    //   their position within that block

    notes = _.map(notes, function (note) {
      const blockId = note.path[0]
      const blockPosition = container.getPosition(blockId)
      const nodePosition = note.start.offset

      return {
        id: note.id,
        'note-content': note['note-content'],
        blockPosition: blockPosition,
        nodePosition: nodePosition
      }
    })

    return _.sortBy(notes, ['blockPosition', 'nodePosition'])
  }

  sortNodesMini (nodes) {
    let notes = _.clone(nodes)
    if (this.config.miniEditorSession === '') return

    notes = _.map(notes, function (note) {
      const element = document.querySelector('p[data-path="' + note.id + '.content"]')
      const noteContent = '<isolated-note data-id="' + note.id + '" data-type="isolated-note">' + element.innerHTML + '</isolated-note>'

      return {
        id: note.id,
        content: noteContent,
        parentNoteId: note.parentNoteId
      }
    })
    return notes
  }

  showNote (note) {
    const notes = this.computeEntries()
    const miniNote = _.find(notes.mini, ['parentNoteId', note.id])
    if (miniNote) this.config.miniEditorContext.editor.emit('noteSelected', miniNote.id)
  }

  createNodeData (note) {
    return {
      'type': 'isolated-note',
      'content': note['note-content'],
      'parentNoteId': note.id
    }
  }

  createIsolatedNote (note) {
    const notes = this.computeEntries()

    if (notes.mini && notes.mini.length > 0) {
      const containerId = this.getMiniContainerId()
      let surface = this.config.miniEditorContext.surfaceManager.getSurface(containerId)
      let container = surface.getContainer()

      let findIndex = _.findIndex(notes.main, ['id', note.id])

      let nodeData = this.createNodeData(note)
      this.config.miniEditorSession.transaction(function (tx) {
        if (!tx.get(nodeData.id)) {
          let blockNode = tx.create(nodeData)
          tx.update(container.getContentPath(), { type: 'insert', pos: findIndex, value: blockNode.id })
        }
      })
    } else {
      this.config.miniEditorSession.transaction(function (tx) {
        let nodeData = this.createNodeData(note)
        tx.insertBlockNode(nodeData)
      }.bind(this))
    }
  }

  getMiniContainerId () {
    return this.config.miniEditorContext.editor.props.containerId
  }
}

NotesProvider.tocTypes = ['note']

export default NotesProvider
